package com.yanzhu.system.service.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.stream.Collectors;

import com.yanzhu.common.core.constant.CacheConstants;
import com.yanzhu.common.core.utils.StringUtils;
import com.yanzhu.common.redis.service.RedisService;
import com.yanzhu.system.domain.vo.TreeSelect;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.yanzhu.system.mapper.SysAreaMapper;
import com.yanzhu.system.domain.SysArea;
import com.yanzhu.system.service.ISysAreaService;

import javax.annotation.PostConstruct;

/**
 * 地址管理Service业务层处理
 * 
 * @author JiangYuQi
 * @date 2024-07-15
 */
@Service
public class SysAreaServiceImpl implements ISysAreaService 
{
    @Autowired
    private RedisService redisService;
    
    @Autowired
    private SysAreaMapper sysAreaMapper;

    /**
     * 项目启动时，初始化参数到缓存
     */
    @PostConstruct
    public void init()
    {
        loadingSysAreaCache();
    }

    /**
     * 加载地址信息到缓存
     */
    @Override
    public void loadingSysAreaCache(){
        SysArea area = new SysArea();
        List<TreeSelect> treeList = selectAreaTreeList(area);
        redisService.setCacheObject(CacheConstants.SYS_AREA_ALL,treeList);
    }

    /**
     * 查询地址管理
     * 
     * @param id 地址管理主键
     * @return 地址管理
     */
    @Override
    public SysArea selectSysAreaById(String id)
    {
        return sysAreaMapper.selectSysAreaById(id);
    }

    /**
     * 查询地址管理列表
     * 
     * @param sysArea 地址管理
     * @return 地址管理
     */
    @Override
    public List<SysArea> selectSysAreaList(SysArea sysArea)
    {
        return sysAreaMapper.selectSysAreaList(sysArea);
    }

    /**
     * 查询区域树结构信息
     *
     * @param area 区域信息
     * @return 区域树信息集合
     */
    @Override
    public List<TreeSelect> selectAreaTreeList(SysArea area)
    {
        List<SysArea> areas = sysAreaMapper.selectSysAreaList(area);
        return buildAreaTreeSelect(areas);
    }
    
    /**
     * 构建前端所需要树结构
     *
     * @param areas 区域列表
     * @return 树结构列表
     */
    @Override
    public List<SysArea> buildAreaTree(List<SysArea> areas)
    {
        List<SysArea> returnList = new ArrayList<SysArea>();
        List<Long> tempList = areas.stream().map(SysArea::getId).collect(Collectors.toList());
        for (SysArea area : areas)
        {
            // 如果是顶级节点, 遍历该父节点的所有子节点
            if (!tempList.contains(area.getParentId()))
            {
                recursionFn(areas, area);
                returnList.add(area);
            }
        }
        if (returnList.isEmpty())
        {
            returnList = areas;
        }
        return returnList;
    }

    /**
     * 构建前端所需要下拉树结构
     *
     * @param areas 区域列表
     * @return 下拉树结构列表
     */
    @Override
    public List<TreeSelect> buildAreaTreeSelect(List<SysArea> areas)
    {
        List<SysArea> areaTrees = buildAreaTree(areas);
        return areaTrees.stream().map(TreeSelect::new).collect(Collectors.toList());
    }
    
    /**
     * 新增地址管理
     * 
     * @param sysArea 地址管理
     * @return 结果
     */
    @Override
    public int insertSysArea(SysArea sysArea)
    {
        return sysAreaMapper.insertSysArea(sysArea);
    }

    /**
     * 修改地址管理
     * 
     * @param sysArea 地址管理
     * @return 结果
     */
    @Override
    public int updateSysArea(SysArea sysArea)
    {
        return sysAreaMapper.updateSysArea(sysArea);
    }

    /**
     * 批量删除地址管理
     * 
     * @param ids 需要删除的地址管理主键
     * @return 结果
     */
    @Override
    public int deleteSysAreaByIds(String[] ids)
    {
        return sysAreaMapper.deleteSysAreaByIds(ids);
    }

    /**
     * 删除地址管理信息
     * 
     * @param id 地址管理主键
     * @return 结果
     */
    @Override
    public int deleteSysAreaById(String id)
    {
        return sysAreaMapper.deleteSysAreaById(id);
    }

    /**
     * 递归列表
     */
    private void recursionFn(List<SysArea> list, SysArea t)
    {
        // 得到子节点列表
        List<SysArea> childList = getChildList(list, t);
        t.setChildren(childList);
        for (SysArea tChild : childList)
        {
            if (hasChild(list, tChild))
            {
                recursionFn(list, tChild);
            }
        }
    }

    /**
     * 得到子节点列表
     */
    private List<SysArea> getChildList(List<SysArea> list, SysArea t)
    {
        List<SysArea> tlist = new ArrayList<SysArea>();
        Iterator<SysArea> it = list.iterator();
        while (it.hasNext())
        {
            SysArea n = (SysArea) it.next();
            if (StringUtils.isNotNull(n.getParentId()) && n.getParentId().longValue() == t.getId().longValue())
            {
                tlist.add(n);
            }
        }
        return tlist;
    }

    /**
     * 判断是否有子节点
     */
    private boolean hasChild(List<SysArea> list, SysArea t)
    {
        return getChildList(list, t).size() > 0 ? true : false;
    }
}
